"""
Service Lookup
==============

Use DNS SRV records to discover services by name and protocol.

"""
from collections import namedtuple
import logging
from dns import resolver
import socket

__version__ = (0, 1, 0)
version = '%s.%s.%s' % __version__

LOGGER = logging.getLogger(__name__)

SRV = namedtuple('SRV', ['host', 'port', 'priority', 'weight'])


class SRVQueryFailure(Exception):
    """Exception that is raised when the DNS query has failed."""
    def __str__(self):
        return 'SRV query failure: %s' % self.args[0]


def lookup(name, protocol='TCP', domain=None):
    """Return a list of service records and associated data for the given
    service name, protocol and optional domain. If protocol is not specified,
    TCP will be used. If domain is not specified, the domain name returned by
    the operating system will be used.

    Service records will be returned as a named tuple with host, port, priority
    and weight attributes:

        >>> import srvlookup
        >>> srvlookup.lookup('api', 'memcached')
        [SRV(host='192.169.1.100', port=11211, priority=1, weight=0),
         SRV(host='192.168.1.102', port=11211, priority=1, weight=0),
         SRV(host='192.168.1.120', port=11211, priority=1, weight=0),
         SRV(host='192.168.1.126', port=11211, priority=1, weight=0)]
        >>>

    :param str name: The service name
    :param str protocol: The protocol name, defaults to TCP
    :param str domain: The domain name to use, defaults to local domain name
    :rtype: list of srvlookup.SRV

    """
    answer = _query_srv_records('_%s._%s.%s' % (name, protocol,
                                                domain or _get_domain()))
    return sorted([SRV(str(record.target).rstrip('.'),
                       record.port,
                       record.priority,
                       record.weight) for record in answer],
                  key=lambda r: r.priority)

def _get_domain():
    """Return the domain name for the local host.

    :rtype: str

    """
    return '.'.join(socket.getfqdn().split('.')[1:])


def _query_srv_records(fqdn):
    """Query DNS for the SRV records of the fully-qualified domain name
    specified.

    :param str fqdn: The fully-qualified domain name to query
    :rtype: dns.resolver.Answer
    :raises: SRVQueryFailure

    """
    try:
        return resolver.query(fqdn, 'SRV')
    except (resolver.NoAnswer,
            resolver.NoNameservers,
            resolver.NotAbsolute,
            resolver.NoRootSOA,
            resolver.NXDOMAIN) as error:
        LOGGER.error('Error querying SRV for %s: %r', fqdn, error)
        raise SRVQueryFailure(error.__class__.__name__)



import mock
from dns import message, name, resolver
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import srvlookup


class WhenRaisingException(unittest.TestCase):

    def test_should_output_exception_string(self):
        msg = 'NXDOMAIN'
        x = srvlookup.SRVQueryFailure(msg)
        self.assertEqual(str(x)[-len(msg):], msg)


class WhenLookingUpRecords(unittest.TestCase):

    MESSAGE = """\
id 1234
opcode QUERY
rcode NOERROR
flags QR AA RD
;QUESTION
foo.bar.baz. IN SRV
;ANSWER
foo.bar.baz. 0    IN  SRV 2 0 11211 1.2.3.4.
foo.bar.baz. 0    IN  SRV 1 0 11211 1.2.3.5.
"""

    def test_should_return_a_list_of_records(self):
        with mock.patch('dns.resolver.query') as query:
            query_name = name.from_text('foo.bar.baz.')
            msg = message.from_text(self.MESSAGE)
            answer = resolver.Answer(query_name,
                                     33, 1, msg,
                                     msg.answer[0])
            query.return_value = answer
            self.assertEqual(srvlookup.lookup('foo', 'bar', 'baz'),
                             [srvlookup.SRV('1.2.3.5', 11211, 1, 0),
                              srvlookup.SRV('1.2.3.4', 11211, 2, 0)])

    def test_should_include_local_domain_when_omitted(self):

        with mock.patch('dns.resolver.query') as query:
            with mock.patch('socket.getfqdn') as getfqdn:
                getfqdn.return_value = 'baz'
                query_name = name.from_text('foo.bar.baz.')
                msg = message.from_text(self.MESSAGE)
                answer = resolver.Answer(query_name,
                                         33, 1, msg,
                                         msg.answer[0])
                query.return_value = answer
                self.assertEqual(srvlookup.lookup('foo', 'bar'),
                                 [srvlookup.SRV('1.2.3.5', 11211, 1, 0),
                                  srvlookup.SRV('1.2.3.4', 11211, 2, 0)])


class WhenInvokingGetDomain(unittest.TestCase):

    EXPECTATION = 'bar.baz.qux'
    VALUE = 'foo.bar.baz.qux'

    def test_should_return_domain_part_only(self):
        with mock.patch('socket.getfqdn') as getfqdn:
            getfqdn.return_value = self.VALUE
            self.assertEqual(srvlookup._get_domain(), self.EXPECTATION)


class WhenInvokingQuerySRVRecords(unittest.TestCase):

    def test_invalid_should_raise_srv_query_failure(self):
        with mock.patch('dns.resolver.query') as query:
            query.side_effect = resolver.NXDOMAIN()
            self.assertRaises(srvlookup.SRVQueryFailure,
                              srvlookup._query_srv_records,
                              'foo.bar.baz')

    def test_resolver_query_should_be_invoked_with_fqdn(self):
        with mock.patch('dns.resolver.query') as query:
            query.return_value = mock.Mock('dns.resolver.Answer')
            fqdn = 'foo.bar.baz'
            srvlookup._query_srv_records(fqdn)
            query.assert_called_once_with(fqdn, 'SRV')

    def test_should_return_resolver_answer(self):
        with mock.patch('dns.resolver.query') as query:
            answer = mock.Mock('dns.resolver.Answer')
            query.return_value = answer
            self.assertEqual(srvlookup._query_srv_records('foo.bar.baz'),
                             answer)



import setuptools
import sys

tests_require = ['nose', 'mock']
if sys.version_info < (2, 7, 0):
    tests_require.append('unittest2')

dnspython = 'dnspython' if sys.version_info < (3, 0, 0) else 'dnspython3'

setuptools.setup(name='srvlookup',
                 version='0.1.0',
                 description='Service lookup using DNS SRV records',
                 long_description=open('README.rst').read(),
                 author='Gavin M. Roy',
                 author_email='gavinr@aweber.com',
                 url='http://github.com/AWeber/srvlookup',
                 py_modules=['srvlookup'],
                 package_data={'': ['LICENSE', 'README.rst']},
                 include_package_data=True,
                 install_requires=[dnspython],
                 tests_require=tests_require,
                 test_suite='nose.collector',
                 license=open('LICENSE').read(),
                 classifiers=['Development Status :: 4 - Beta',
                              'Intended Audience :: Developers',
                              'License :: OSI Approved :: BSD License',
                              'Operating System :: OS Independent',
                              'Programming Language :: Python :: 2',
                              'Programming Language :: Python :: 2.6',
                              'Programming Language :: Python :: 2.7',
                              'Programming Language :: Python :: 3',
                              'Programming Language :: Python :: 3.2',
                              'Programming Language :: Python :: 3.3',
                              'Programming Language :: Python :: Implementation :: CPython',
                              'Programming Language :: Python :: Implementation :: PyPy',
                              'Topic :: Communications',
                              'Topic :: Internet',
                              'Topic :: Software Development :: Libraries'],
                 zip_safe=True)